#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::submitrecord;

use strict;
use warnings;

require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(submitrecord);

our $VERSION = '0.01';

#######################################################################
our @ARGVBAACKUP = @ARGV;
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => [ ], 'cmdname'  => "submit");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
			 'user'		=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our	%FieldDef = ();
our     @fieldOrder = ();
our	%DbIdDef = ();
our	%RIdDef = ();;
our	$TypeDef = "";
our	$DuplicateRid=undef;
our	$ActionDef = undef;

#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "submit");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	@fieldOrder = ();
	%FieldDef = ();
	%DbIdDef = ();
	%RIdDef = ();
      $TypeDef = "";
	$DuplicateRid=undef;
	$ActionDef = "submit";
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => [ ], 'cmdname'  => "submit");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	@fieldOrder = ();				
	%FieldDef = ();
	%DbIdDef = ();
	%RIdDef = ();
      $TypeDef = getDefaultRecordtype();
	$DuplicateRid=undef;
	$ActionDef = undef;
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;
}

#########handle getopt warnings end#####################################

sub handle_help_usage
{
	my $opt = $_[0];
	
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}


sub handle_field_list
{
	my ($cnt , @tmpARGV) = @_;
	my $msg = undef;
	my $fieldvaluelist = undef;

	if(exists $tmpARGV[$cnt+1] && (!($tmpARGV[$cnt+1] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+1] =~ /^(-)(.+)/)))
	{

		my $i = 3;
		if(exists $tmpARGV[$cnt+2])
		{
			if(!($tmpARGV[$cnt+2] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+2] =~ /^(-)(.+)/)){
				$fieldvaluelist  = $tmpARGV[$cnt+2];
			}
			else {
				ThrowSyntaxErrorException('CQCLI_Missing_FValue',$tmpARGV[$cnt+1],$CmdInfo{'cmdname'});
			}

			while(exists $tmpARGV[$cnt+$i]){

				if(!($tmpARGV[$cnt+$i] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+$i] =~ /^(-)(.+)/))
				{
					$fieldvaluelist = $fieldvaluelist . chr(7) . $tmpARGV[$cnt+$i];$i++;
				}
				else {
				goto NEXT;
				}
			}
			NEXT:$FieldDef{$tmpARGV[$cnt+1]} =  $fieldvaluelist;
			# Get the size of the hash and that will be the last index where the new key has to be added in the array
			$fieldOrder[(keys(%FieldDef))-1] = $tmpARGV[$cnt+1];
			splice @ARGV,0,$i-1;
			return $i-1;

		} else {
			ThrowSyntaxErrorException('CQCLI_Missing_FValue',$tmpARGV[$cnt+1],$CmdInfo{'cmdname'});
		}
	} else {
		ThrowSyntaxErrorException('CQCLI_Missing_FName',$tmpARGV[$cnt],$CmdInfo{'cmdname'});
	}

	return 1;
}

sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}


sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForInterActiveOption 
{
	my $ret = undef;
	my $askRules = {'ask!'	=>	\&handle_ask_mode	};
	Getopt::Long::Configure('posix_default','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$askRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}

sub parseForCoreOptions()
{
	my $ret = undef;
	my $CoreOptRules = {
		'type=s' =>	\$TypeDef			 
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForUnknownOptions
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	my $msg = undef;

	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	

		if( $tmpARGV[$cnt] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i )
		{
			my $skip = handle_field_list($cnt,@tmpARGV);
			$cnt=$cnt+$skip;
			next;
		}elsif($tmpARGV[$cnt] =~ /^(-)(.+)/)
		{
			my $fname = $2;
			my $i = 2;
			my $fieldvaluelist = undef;

			if( exists $tmpARGV[$cnt+1] && !($tmpARGV[$cnt+1] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+1] =~ /^(-)(.+)/)){
			   $fieldvaluelist = $tmpARGV[$cnt+1];
			}
			else {	
				ThrowSyntaxErrorException('CQCLI_Missing_Value',$tmpARGV[$cnt],$CmdInfo{'cmdname'});
			}

			while( exists $tmpARGV[$cnt+$i]){

				if(!($tmpARGV[$cnt+$i] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+$i] =~ /^(-)(.+)/)){
					$fieldvaluelist = $fieldvaluelist . chr(7) . $tmpARGV[$cnt+$i]; 
				}
				else {
				goto NEXT;
				}
				$i++;
			}
			
			NEXT:$cnt=$cnt+($i-1);
			splice @ARGV,0,$i;
			# Get the size of the hash and that will be the last index where the new key has to be added in the array
			$fieldOrder[keys(%FieldDef)] = $fname;
			$FieldDef{$fname}=$fieldvaluelist;
		}
		else{
			ThrowSyntaxErrorException('CQCLI_Invalid_Option',$tmpARGV[$cnt]);
		}
		
	}
	return 1;
}

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}elsif($CliSession){
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}else{
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
	      $JavaReqObj->put("type",$TypeDef);
		return;
	}

}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}

sub ValidateFieldNames()
{

	my @actualfieldlist = keys %FieldDef;
	my $recordtypeobj = undef;
	my $recordinfoobj = undef;
	my $mflist = undef;
	my @mFlist = ();
	my @resultantmFlist = ();
	my ($key,$fhash)=(undef,undef);
	my %fldhash = ();

	if($#actualfieldlist >= 0) {
		ValidateExistingFields(@actualfieldlist);
	}
	if(IsAskMode())
	{
		
			if(AskForYN('CQCLI_Prompt_AskFieldName')) {
			$fhash = GetAllFieldsInterActively($TypeDef,undef,1);
			%fldhash = ();
			if($fhash){
				%fldhash = %{$fhash};
				foreach $key (keys %{$fhash}) {
					# Get the size of the hash and that will be the last index where the new key has to be added in the array
					$fieldOrder[keys(%FieldDef)] = $key;
					$FieldDef{$key} = $fldhash{$key};
					
				}
			}
		}
		SetCQCLIAskMode(0);
	}

	my ($alist,$mfname,$idtype)=(undef,undef,undef,undef);
	my ($ridlist,$fieldm) = (undef,undef);
	
	@actualfieldlist = keys %FieldDef;
	$recordtypeobj = GetRecordTypeInfoObj($TypeDef);
      $ActionDef = $recordtypeobj->getSubmitActionName();
	$mflist = $recordtypeobj->getMandatoryFieldWithAction($ActionDef);

	if($mflist) {
		@mFlist = split(/\,/,$mflist);
	}
	#for no mandatory fields
	if($#mFlist < 0) {
		goto CONTINUE;
		return 1;
	}
	$alist = join(",",@actualfieldlist);
	$alist = ",".$alist.",";
	foreach  $mfname (@mFlist) {
		if($alist =~ /\,$mfname\,/i) { 
			next;
			}
		else{
			push @resultantmFlist , $mfname;
		}
	}
	
	if(length(@resultantmFlist) == 0) {
	goto CONTINUE;
	}
	
	foreach  $fieldm (@resultantmFlist) {
			
			if(!CheckPromptModeValidity('CQCLI_Missing_MandatoryFields')){
				$FieldDef{$fieldm} = GetFieldValueInterActively($fieldm,$TypeDef,undef,1);
				# Get the size of the hash and that will be the last index where the new key has to be added in the array
				$fieldOrder[(keys(%FieldDef))-1] = $fieldm;
			}
	}
	
	CONTINUE:
	SetFieldNames();
	
}


sub ValidateExistingFields
{
	my @fieldlist = @_;
	my $actualfield = undef;
	my $expectedfield = undef;
	my @field = undef;

	my $recordtypeobj = GetRecordTypeInfoObj($TypeDef);

	foreach $actualfield (@fieldlist) {
		$expectedfield = $recordtypeobj->isValidFieldName($actualfield);
		if(!($expectedfield)) {
			ThrowValidationException('CQCLI_Failure_Field',$actualfield,$TypeDef);
		}elsif($expectedfield =~ /\,/){
			ThrowValidationException('CQCLI_Failure_FieldAmbiguity',$actualfield,$expectedfield,$TypeDef);
		}else{
			if($actualfield ne $expectedfield){
				$FieldDef{$expectedfield}=$FieldDef{$actualfield};
				delete $FieldDef{$actualfield};
				$actualfield = $expectedfield;
			}

			#ValidateFieldValue($actualfield,$FieldDef{$actualfield},$TypeDef,$recordids[0],$idtype);
		}
	}
}

sub SetFieldNames()
{
	#set request object for successfull field validation
	my $fld = undef;
	my $counter = 0;
	my $key = undef;
	my $tempVar = $CliSession->createOrderedMap();;

	foreach ( @fieldOrder ) {
		if (exists $fieldOrder[$counter]) {
			$key = $fieldOrder[$counter];
			my $tempValue = $FieldDef{$key};
			$tempVar->put($key, $FieldDef{$key}) ;	
		
		}
		$counter++;		
	}
	$JavaReqObj->put("field",$tempVar);

}

sub Parse()
{
	try {
		parseForHelpUsage();
		parseForInterActiveOption();
		parseForLoginCredentials();
		parseForCoreOptions();
		parseForUnknownOptions();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Validate()
{
	try {
		ValidateSessionInfo();
		ValidateFieldNames();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub submitrecord
{
	my @tmpCmd = @_;
	my $req = undef;
	try {

		InitParser(@tmpCmd);
		Parse();
		Validate();
		$req =  GetJavaPayload("submit",$JavaReqObj);
		SetRequestObject($req);
		return 1;
	}finally {
		CleanUpParser();
	};
}

1;
